#pragma once
#include "StdAfx.h"
#include "DesignElementVis.h"


float DesignElementVis::mag_ratio = 0.04f;
const icVector2 DesignElementVis::bx(1.0, 0.0);
const icVector2 DesignElementVis::by(0.0, 1.0);


void DesignElementVis::draw_empty_rectangle(icVector2 center, icVector2 min_dir, icVector2 max_dir, float min_mag, float max_mag, int line_width, int style, icVector3 color) {
	glColor3d(color.entry[0], color.entry[1], color.entry[2]);
	glLineWidth((GLfloat)line_width);

	glBegin(GL_LINES);
	glVertex2d(center.entry[0]-mag_ratio*max_mag*max_dir.entry[0]-mag_ratio*min_mag*min_dir.entry[0], 
						 center.entry[1]-mag_ratio*max_mag*max_dir.entry[1]-mag_ratio*min_mag*min_dir.entry[1]);
	glVertex2d(center.entry[0]+mag_ratio*max_mag*max_dir.entry[0]-mag_ratio*min_mag*min_dir.entry[0], 
						 center.entry[1]+mag_ratio*max_mag*max_dir.entry[1]-mag_ratio*min_mag*min_dir.entry[1]);
	glEnd();


	glBegin(GL_LINES);
	glVertex2d(center.entry[0]+mag_ratio*max_mag*max_dir.entry[0]-mag_ratio*min_mag*min_dir.entry[0], 
						 center.entry[1]+mag_ratio*max_mag*max_dir.entry[1]-mag_ratio*min_mag*min_dir.entry[1]);
	glVertex2d(center.entry[0]+mag_ratio*max_mag*max_dir.entry[0]+mag_ratio*min_mag*min_dir.entry[0], 
						 center.entry[1]+mag_ratio*max_mag*max_dir.entry[1]+mag_ratio*min_mag*min_dir.entry[1]);
	glEnd();
	
	glBegin(GL_LINES);
	glVertex2d(center.entry[0]+mag_ratio*max_mag*max_dir.entry[0]+mag_ratio*min_mag*min_dir.entry[0], 
						 center.entry[1]+mag_ratio*max_mag*max_dir.entry[1]+mag_ratio*min_mag*min_dir.entry[1]);
	glVertex2d(center.entry[0]-mag_ratio*max_mag*max_dir.entry[0]+mag_ratio*min_mag*min_dir.entry[0], 
						 center.entry[1]-mag_ratio*max_mag*max_dir.entry[1]+mag_ratio*min_mag*min_dir.entry[1]);
	glEnd();

	glBegin(GL_LINES);
	glVertex2d(center.entry[0]-mag_ratio*max_mag*max_dir.entry[0]+mag_ratio*min_mag*min_dir.entry[0], 
						 center.entry[1]-mag_ratio*max_mag*max_dir.entry[1]+mag_ratio*min_mag*min_dir.entry[1]);
	glVertex2d(center.entry[0]-mag_ratio*max_mag*max_dir.entry[0]-mag_ratio*min_mag*min_dir.entry[0], 
						 center.entry[1]-mag_ratio*max_mag*max_dir.entry[1]-mag_ratio*min_mag*min_dir.entry[1]);
	glEnd();
}


void DesignElementVis::draw_filled_circle(icVector2 center, icVector2 min_dir, icVector2 max_dir, float radius, icVector3 color) {
	int num_edges = 24;
	double theta; 
	float new_radius = radius * mag_ratio;
	icVector2 temp;

	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glDisable(GL_LIGHTING);
	glColor3d(color.entry[0], color.entry[1], color.entry[2]);
	glBegin(GL_POLYGON);

	for(int i = 0; i < num_edges; i++) {
		theta = i * 2 * M_PI / num_edges;
		temp = new_radius * cos(theta) * max_dir + new_radius * sin(theta) * min_dir + center;	
		glVertex2d(temp.entry[0], temp.entry[1]);
	}

	glEnd();
}


void DesignElementVis::drawSquareKnob(const SingularElement &se, icVector3 color, bool is_active, float desired_width) {
	draw_empty_rectangle(se.center, se.min_dir, se.max_dir, (float)(se.min_mag), (float)(se.max_mag), (int)desired_width, 0, color);

	if(is_active == false) return;

	// this part shows the rotation knob and scaling knobs
	glLineWidth((GLfloat)desired_width);
	glColor3d(color.entry[0], color.entry[1], color.entry[2]);
	glBegin(GL_LINES);
	glVertex2d(se.stretch_center[2].entry[0], se.stretch_center[2].entry[1]);
	glVertex2d(se.rotate_center.entry[0], se.rotate_center.entry[1]);
	glEnd();

	for(int i = 0; i < 8; i++) {
		draw_filled_circle(se.stretch_center[i], bx, by, 0.15f, color);
	}

	draw_filled_circle(se.rotate_center, bx, by, 0.15f, color);
}


void DesignElementVis::drawArrow(const RegularElement &re, icVector3 color, bool is_active, bool draw_both_ends, float desired_width) {
	glColor3d(color.entry[0], color.entry[1], color.entry[2]);
	glBegin(GL_LINES);
	glVertex2d(re.start_center.entry[0], re.start_center.entry[1]);
	glVertex2d(re.end_center.entry[0], re.end_center.entry[1]);

	if(draw_both_ends == true) {
		glVertex2d(re.start_center.entry[0], re.start_center.entry[1]);
		glVertex2d(re.other_end_center.entry[0], re.other_end_center.entry[1]);
	}

	// draw arrow
	glVertex2d(re.end_center.entry[0], re.end_center.entry[1]);
	glVertex2d(re.side_pt[0].entry[0], re.side_pt[0].entry[1]);

	glVertex2d(re.end_center.entry[0], re.end_center.entry[1]);
	glVertex2d(re.side_pt[1].entry[0], re.side_pt[1].entry[1]);

	if(draw_both_ends == true) {
		glVertex2d(re.other_end_center.entry[0], re.other_end_center.entry[1]);
		glVertex2d(re.side_pt[2].entry[0], re.side_pt[2].entry[1]);

		glVertex2d(re.other_end_center.entry[0], re.other_end_center.entry[1]);
		glVertex2d(re.side_pt[3].entry[0], re.side_pt[3].entry[1]);
	}
	
	glEnd();

	if(is_active == false) return;

	// this part shows the knobs
	draw_filled_circle(re.start_center, bx, by, 0.15f, color);
	draw_filled_circle(re.end_center, bx, by, 0.15f, color);
}


void DesignElementVis::drawSeparatrix(Singularity &sin, icVector3 color_major, icVector3 color_minor, int major_minor, float desired_width) {
	int num_seps, num_ls;
	LineSegmentList *sep_lst;
	LineSegment *ls;


	glLineWidth((GLfloat)desired_width);
	glBegin(GL_LINES);

	if((major_minor & 1) != 0) {	// major
		num_seps = sin.separatrices_major->num_separatrices;

		glColor3d(color_major.entry[0], color_major.entry[1], color_major.entry[2]);

		for(int i = 0; i < num_seps; i++) {
			sep_lst = sin.separatrices_major->separatrices[i];
			num_ls = sep_lst->num_linesegments;

			for(int j = 0; j < num_ls; j++) {
				ls = sep_lst->linesegments[j];
				glVertex2d(ls->start_pos.entry[0], ls->start_pos.entry[1]);
				glVertex2d(ls->end_pos.entry[0], ls->end_pos.entry[1]);
			}
		}
	}

	if((major_minor & 2) != 0) {	// minor
		num_seps = sin.separatrices_minor->num_separatrices;

		glColor3d(color_minor.entry[0], color_minor.entry[1], color_minor.entry[2]);

		for(int i = 0; i < num_seps; i++) {
			sep_lst = sin.separatrices_minor->separatrices[i];
			num_ls = sep_lst->num_linesegments;

			for(int j = 0; j < num_ls; j++) {
				ls = sep_lst->linesegments[j];
				glVertex2d(ls->start_pos.entry[0], ls->start_pos.entry[1]);
				glVertex2d(ls->end_pos.entry[0], ls->end_pos.entry[1]);
			}
		}
	}

	glEnd();
}


void DesignElementVis::drawSingularity(Singularity &sin, icVector3 color) {
	draw_filled_circle(sin.center, bx, by, 0.15f, color);
}

