#pragma once
#include "stdafx.h"
#include "OpticalFlowReader.h"


using namespace std;



OpticalFlowReader::OpticalFlowReader() {
	w = -1;
	h = -1;
	num_frames = 0;
	direction = -1;
	optflow = NULL;
}


OpticalFlowReader::~OpticalFlowReader() {
	if(optflow) {
		fclose(optflow);
	}
}


void OpticalFlowReader::clear() {
	if(optflow != NULL) {
		fclose(optflow);
		optflow = NULL;
		w = -1;
		h = -1;
		num_frames = 0;
		direction = -1;
	}
}


bool OpticalFlowReader::read(const TCHAR *filepath) {

	if(filepath == NULL) return false;

	clear();

	optflow = _tfopen(filepath, _T("rb"));

	if(optflow != NULL) {
		fscanf(optflow, "%d %d %d %d", &w, &h, &num_frames, &direction);
		return true;
	}
	else {
		return false;
	}
}


int OpticalFlowReader::getWidth() {
	return w;
}


int OpticalFlowReader::getHeight() {
	return h;
}


int OpticalFlowReader::getNumFrames() {
	return num_frames;
}


void OpticalFlowReader::getDimension(int &width, int &height) {
	width = w;
	height = h;
}


int OpticalFlowReader::getOpticalFlowDirection() {
	return direction;
}


bool OpticalFlowReader::load(int frame, CvPoint2D32f *data) {
	int idx;
	int num_elems = w * h;
	int cnt = 0;
	long offset = 0;

	int dum[4];
	char dum2;

	if(data == NULL || optflow == NULL) return false;

	if(0 <= frame && frame < num_frames) {
		if((direction == 1 && frame == num_frames - 1) || (direction == 0 && frame == 0)) {
			for(int i = 0; i < h; i++) {
				for(int j = 0; j < w; j++) {
					idx = j + w * i;
					data[idx].x = (float)j;
					data[idx].y = (float)i;
				}
			}
		}
		else {
			fseek(optflow, 0, SEEK_SET);
			fscanf(optflow, "%d %d %d %d", &(dum[0]), &(dum[1]), &(dum[2]), &(dum[3]));		// read header again...
			fread(&dum2, sizeof(char), 1, optflow);

			switch(direction) {
				case 0:
					offset = (frame - 1) * num_elems * 2 * sizeof(float);
					break;

				case 1:
					offset = frame * num_elems * 2 * sizeof(float);
					break;
			}

			fseek(optflow, offset, SEEK_CUR);

			for(int i = 0; i < num_elems; i++) {
				fread(&(data[i]), sizeof(float), 2, optflow);
			}
		}
	}
	else {
		for(int i = 0; i < h; i++) {
			for(int j = 0; j < w; j++) {
				idx = j + w * i;
				data[idx].x = (float)j;
				data[idx].y = (float)i;
			}
		}
	}

	return true;
}


bool OpticalFlowReader::load(int frame, float *data) {
	int idx;
	int num_elems = w * h;
	int cnt = 0;
	long offset = 0;

	int dum[4];
	char dum2;

	if(data == NULL || optflow == NULL) return false;

	if(0 <= frame && frame < num_frames) {
		if((direction == 1 && frame == num_frames - 1) || (direction == 0 && frame == 0)) {
			idx = 0;

			for(int i = 0; i < h; i++) {
				for(int j = 0; j < w; j++) {
					data[idx++] = (float)j;
					data[idx++] = (float)i;
				}
			}
		}
		else {
			fseek(optflow, 0, SEEK_SET);
			fscanf(optflow, "%d %d %d %d", &(dum[0]), &(dum[1]), &(dum[2]), &(dum[3]));		// read header again...
			fread(&dum2, sizeof(char), 1, optflow);

			switch(direction) {
				case 0:
					offset = (frame - 1) * num_elems * 2 * sizeof(float);
					break;

				case 1:
					offset = frame * num_elems * 2 * sizeof(float);
					break;
			}

			fseek(optflow, offset, SEEK_CUR);
			fread(data, sizeof(float), 2 * num_elems, optflow);
		}
	}
	else {
		idx = 0;

		for(int i = 0; i < h; i++) {
			for(int j = 0; j < w; j++) {
				data[idx++] = (float)j;
				data[idx++] = (float)i;
			}
		}
	}

	return true;
}
