#pragma once
#include "StdAfx.h"
#include "PaintingRegion.h"


using namespace std;


#define IS_GLOBAL_REGION_ID(x) (x == -1)


const float PaintingRegion::highlight_color[] = {0.f, 0.f, 0.f, 0.f};
const float PaintingRegion::dehighlight_color[] = {0.f, 0.f, 0.f, 0.4f};



PaintingRegion::PaintingRegion(int id) : t_preset() {
	this->id = id;

	use_zero_field = false;
	apply_transformation = true;
	num_frames = 0;
	diffusion = true;
	current_preset = PRESET_NONE;
	setDesignElemHolderSize(1);
}


bool PaintingRegion::setKey(PaintingPreset p, int t) {
	current_preset = p;

	return t_preset.set(p, t);
}


PaintingPreset PaintingRegion::getKey(int t) {
	current_preset = t_preset.get(t);

	return current_preset;
}


int PaintingRegion::getNextKey(int t) {
	return t_preset.getNextKey(t);
}


int PaintingRegion::getPrevKey(int t) {
	return t_preset.getPrevKey(t);
}


bool PaintingRegion::removeKey(int t) {
	if(t_preset.remove(t)) {
		current_preset = t_preset.get(t);
		return true;
	}
	else {
		return false;
	}
}


int PaintingRegion::getID() {
	return id;
}


PaintingPreset PaintingRegion::getCurrentKey() {
	return current_preset;
}


void PaintingRegion::setCurrentKey(PaintingPreset p) {
	current_preset = p;
}


void PaintingRegion::copyKeys(const PaintingRegion &src) {
	current_preset = src.current_preset;

	t_preset.copy(src.t_preset);
}


inline bool PaintingRegion::setDesignElemHolderSize(int size, const string &file_path) {
	bool val;

	if(size <= 0) size = 1;

	//diffusion = false;
	diffusion = true;

	if(size > 0) {
		num_frames = size;
		design_elements.resize(size);
		transport.resize(size);

		// construct
		val = buildRegionMotions(file_path);

		return true;
	}
	else {
		num_frames = 1;
		design_elements.resize(1);
		transport.resize(1);
		return false;
	}
}


int PaintingRegion::getNextDEKey(int t) {
	int num_de = (int)design_elements.size();


	for(int i = t + 1; i < num_de; i++) {
		if(design_elements.at(i).numKF() > 0) return i;
	}

	return -1;
}


int PaintingRegion::getPrevDEKey(int t) {
	int num_de = (int)design_elements.size();


	for(int i = t - 1; i >= 0; i--) {
		if(design_elements.at(i).numKF() > 0) return i;
	}

	return -1;
}


bool PaintingRegion::propagateDesignElems(SingularElementList *se, RegularElementList *re) { 
	return propagateDesignElems(se->singularelements, se->num_singularelements, re->regularelements, re->num_regularelements);
}


bool PaintingRegion::propagateDesignElems(SingularElement **se, size_t num_se, RegularElement **re, size_t num_re) {
	int prev_kf = -1;
	int next_kf = -1;


	vector<int> dist_from_prev_kf;
	vector<int> dist_from_next_kf;


	if(num_frames <= 0 || se == NULL || re == NULL) return false;


	// clear up
	for(int i = 0; i < num_frames; i++) {
		design_elements[i].clear();
	}

	diffusion = true;


	// no need to process with empty design element set.
	if(num_se == 0 && num_re == 0) return true;

	dist_from_prev_kf.resize(num_frames);
	dist_from_next_kf.resize(num_frames);



#if DEBUG_SAVE_ELEMENTS_INFO_AS_TEXT == 1
	stringstream d_save_filename;
	d_save_filename.str("");
	d_save_filename << "Orientation_Info_" << setfill('0') << setw(2) << id << ".txt";
	d_fstr_de.open(d_save_filename.str().c_str(), ios_base::out);
#endif


	// fill key frames based on design elements
	for(size_t i = 0; i < num_se; i++) {
		if((se[i]->regionID == this->id || IS_GLOBAL_REGION_ID(se[i]->regionID))) {

			// check if it is within a frame ragne
			if(0 <= se[i]->frameID && se[i]->frameID < num_frames) {
				design_elements[se[i]->frameID].se.kf.push_back(*se[i]);
			}
			else {
			}
		}
	}

	for(size_t i = 0; i < num_re; i++) {
		if(re[i]->regionID == this->id || IS_GLOBAL_REGION_ID(re[i]->regionID)) {

			// check if it is within a frame ragne
			if(0 <= re[i]->frameID && re[i]->frameID < num_frames) {
				design_elements[re[i]->frameID].re.kf.push_back(*re[i]);
			}
			else {
			}
		}
	}


	// step 1: traverse the video forward
	for(int i = 0; i < num_frames; i++) {
		if(isKeyFrameOfDesignElems(i) == true) {
			// a new keyframe is encountered for region j... reset prev_kf
			prev_kf = i;
			dist_from_prev_kf[i] = 0;
			continue;
		}

		// there is a keyframe before this one
		if(prev_kf >= 0) {
			dist_from_prev_kf[i] = dist_from_prev_kf[i - 1] + 1;

			if(i == prev_kf + 1) {
				copyDesignElements(
					design_elements[i - 1].se.kf, design_elements[i].se.fwd_elems, 
					design_elements[i - 1].re.kf, design_elements[i].re.fwd_elems, 
					i - 1, true);
			}
			else {
				copyDesignElements(
					design_elements[i - 1].se.fwd_elems, design_elements[i].se.fwd_elems, 
					design_elements[i - 1].re.fwd_elems, design_elements[i].re.fwd_elems, 
					i - 1, true);
			}
		}
	}


	// step 2: traverse the video backward
	for(int i = num_frames - 1; i >= 0; i--) {
		if(isKeyFrameOfDesignElems(i) == true) {
			next_kf = i;
			dist_from_next_kf[i] = 0;
			continue;
		}

		// there is a keyframe after this one
		if(next_kf >= 0) {
			dist_from_next_kf[i] = dist_from_next_kf[i + 1] + 1;

			if(i == next_kf - 1) {
				copyDesignElements(
					design_elements[i + 1].se.kf, design_elements[i].se.bwd_elems, 
					design_elements[i + 1].re.kf, design_elements[i].re.bwd_elems, 
					i + 1, false);
			}
			else {
				copyDesignElements(
					design_elements[i + 1].se.bwd_elems, design_elements[i].se.bwd_elems, 
					design_elements[i + 1].re.bwd_elems, design_elements[i].re.bwd_elems, 
					i + 1, false);
			}
		}
	}


	// compute weight for each frame
	for(int i = 0; i < num_frames; i++) {
		if(design_elements[i].numKF() > 0) { design_elements[i].weight_forward = -1.f; continue; }		// neither forward nor backward
		if(design_elements[i].numFwd() == 0) { design_elements[i].weight_forward = -1.f; continue; }
		if(design_elements[i].numBwd() == 0) { design_elements[i].weight_forward = -1.f; continue; }

		design_elements[i].weight_forward = dist_from_next_kf[i] / (float)(dist_from_next_kf[i] + dist_from_prev_kf[i]);
	}

	// look at the first frame whose frame ID is zero...
	if(design_elements[0].size() > 0) {
		diffusion = false;
	}
	else  {
		diffusion = true;
	}

	for(int i = 0; i < num_frames; i++) {
		design_elements[i].diffusion = diffusion;
		design_elements[i].region_id = id;
	}


#if DEBUG_SAVE_ELEMENTS_INFO_AS_TEXT == 1

	d_fstr_de.close();
#endif



	/**
	if(this->id == 2) {
		FILE *fp = fopen("blah.txt", "w");
		int lp = min(10, num_frames);
		for(int i = 0; i < lp ; i++) {
			fprintf(fp, "%d, %d, %d, %d, %d\n", design_elements[i].numKF(), design_elements[i].numFwd(), design_elements[i].numBwd(), dist_from_prev_kf[i], dist_from_next_kf[i]);
		}
		fclose(fp);
	}
	/**/


	return true;
}


void PaintingRegion::copyDesignElements(const list<SingularElement> &se_from, list<SingularElement> &se_to, const list<RegularElement> &re_from, list<RegularElement> &re_to, int n1, bool forward) {
	typedef list<SingularElement> sel;
	sel::const_iterator se_itr, se_itr_end;

	typedef list<RegularElement> rel;
	rel::const_iterator re_itr, re_itr_end;

	SingularElement *se;
	RegularElement *re;

	int idx;
	double x0, y0, theta;
	double dx, dy, dtheta;						// translation and orientation
	double ct, st, nx, ny, angle, nx1, ny1;
	double lx, ly, lx1, ly1;
	double mag_ratio = .04;


	if(n1 < 0 || n1 >= num_frames) return;

	if(forward) {
		if(n1 + 1 >= num_frames)  {
			idx = -1;		// no appropriate index
		}
		else {
			idx = n1;
		}
	}
	else {
		if(n1 - 1 < 0)  {
			idx = -1;		// no appropriate index
		}
		else {
			idx = n1 - 1;
		}
	}

	// nothing to copy to.
	if(idx == -1) return;


	if(apply_transformation == true) {
		x0 = transport[idx].x0;
		y0 = transport[idx].y0;
		theta = transport[idx].theta;

		if(forward == true) {
			dx = transport[idx].dx;
			dy = transport[idx].dy;
			dtheta = transport[idx].dtheta;
		}
		else {
			dx = -transport[idx].dx;
			dy = -transport[idx].dy;
			dtheta = -transport[idx].dtheta;
		}
	}
	else {
		x0 = 0;
		y0 = 0;
		theta = 0;

		dx = 0;
		dy = 0;
		dtheta = 0;
	}


#if DEBUG_SAVE_ELEMENTS_INFO_AS_TEXT == 1
	int fwidth = 10;
	d_fstr_de.precision(3);
	d_fstr_de << "@ " << (n1 + 1) << endl;
	d_fstr_de << (forward ? "Forward" : "Backward") << ", Index Used from @ " << (idx + 1) << endl;
	d_fstr_de << "dx: " << setw(fwidth) << dx << ", dy: " << setw(fwidth) << dy << ", dtheta: " << setw(fwidth) << dtheta << endl;

#endif


	// apply to singular elements
	for(se_itr = se_from.begin(), se_itr_end = se_from.end(); se_itr != se_itr_end; se_itr++) {
		se_to.push_back(*se_itr);
		se = &(se_to.back());

		// apply motion only to local elements
		if(se->regionID >= 0) {
			// new angle of element
			angle = atan2(se->max_dir.entry[1], se->max_dir.entry[0]) + dtheta;

			// new position of element
			ct = cos(dtheta);
			st = sin(dtheta);
			lx = se->x - x0;
			ly = se->y - y0;
			nx = lx * ct - ly * st + x0 + dx;
			ny = lx * st + ly * ct + y0 + dy;

			se->Update_Position(nx, ny, 0);
			se->Update_Angle(angle);


#if DEBUG_SAVE_ELEMENTS_INFO_AS_TEXT == 1
			d_fstr_de << "Singular Element" << endl;
			d_fstr_de << "starting point... (x0, y0): (" << setw(fwidth) << lx+x0 << ", " << setw(fwidth) << ly+y0 << ")" << endl;
			d_fstr_de << "ending point...   (x0, y0): (" << setw(fwidth) << nx    << ", " << setw(fwidth) << ny    << ")" << endl;
			d_fstr_de << endl;
#endif
		}
	}


	// apply to regular elements
	for(re_itr = re_from.begin(), re_itr_end = re_from.end(); re_itr != re_itr_end; re_itr++) {
		re_to.push_back(*re_itr);
		re = &(re_to.back());

		// apply motion only to local elements
		if(re->regionID >= 0) {
			ct = cos(dtheta);
			st = sin(dtheta);
			lx = re->x - x0;
			ly = re->y - y0;
			lx1 = re->x + re->vx * mag_ratio - x0;
			ly1 = re->y + re->vy * mag_ratio - y0;

			nx = lx * ct - ly * st + x0 + dx;
			ny = lx * st + ly * ct + y0 + dy;
			re->Update_Start_Position(nx, ny);
			nx1 = lx1 * ct - ly1 * st + x0 + dx;
			ny1 = lx1 * st + ly1 * ct + y0 + dy;
			re->Update_End_Position(nx1, ny1);


#if DEBUG_SAVE_ELEMENTS_INFO_AS_TEXT == 1
			d_fstr_de << "Regular Element" << endl;
			d_fstr_de << "starting point... (x0, y0): (" << setw(fwidth) << lx+x0 << ", " << setw(fwidth) << ly+y0 << "), (x1, y1): (" << setw(fwidth) << lx1+x0 << ", " << setw(fwidth) << ly1+y0 << "), Len: " << setw(fwidth) << sqrt((lx-lx1)*(lx-lx1)+(ly-ly1)*(ly-ly1)) << endl;
			d_fstr_de << "ending point...   (x0, y0): (" << setw(fwidth) << nx    << ", " << setw(fwidth) << ny    << "), (x1, y1): (" << setw(fwidth) << nx1    << ", " << setw(fwidth) << ny1    << "), Len: " << setw(fwidth) << sqrt((nx-nx1)*(nx-nx1)+(ny-ny1)*(ny-ny1)) << endl;
			d_fstr_de << endl;
#endif
		}
	}
}


bool PaintingRegion::buildRegionMotions(const string &file_path) {
	int n1, n2;
	stringstream filename;
	const string suffix = ".dat";
	RegionTransformation rt;
	RegionTransformation::imageDat *img_dat = NULL;
	RegionTransformation::ellipse *ellipse = NULL;
	RegionTransformation::ellipseTransformation *trans = NULL;
	transformation tmp;
	const double rad = M_PI / 180;

	// do nothing...
	if(file_path == string()) return false;


#if DEBUG_SAVE_ELLIPSETRANSF_AS_TEXT == 1
	
	stringstream d_save_filename;
	bool alert_once = true;
	int fwidth = 10;

	d_save_filename.str("");
	d_save_filename << "PaintingRegion_Info_" << setfill('0') << setw(2) << id << ".txt";
	d_fstr_ellip.open(d_save_filename.str().c_str(), ios_base::out);

#endif

	for(int i = 0; i < num_frames; i++) {			//if(i >= 150) break;
		n1 = i;
		n2 = n1 + 1;
		filename.str("");	
		filename << setfill('0') << file_path << setw(3) << n1 << '_' << setw(3) << n2 << suffix;

		if(rt.loadDat(filename.str().c_str()) == true) {
			img_dat = rt.imgdat;

			if(0 < id && id <= img_dat->nRegions) {
				ellipse = &(img_dat->regions[id - 1].eCur);
				trans = &(img_dat->regions[id - 1].eTransf);
				tmp.x0 = ellipse->x / img_dat->width - .5;
				tmp.y0 = .5 - ellipse->y / img_dat->height;
				tmp.theta = ellipse->theta * rad * -1.;
				tmp.dx = trans->tx / img_dat->width;
				tmp.dy = -trans->ty / img_dat->height;
				tmp.dtheta = trans->dTheta * rad * -1.;


#if DEBUG_SAVE_ELLIPSETRANSF_AS_TEXT == 1
				d_fstr_ellip << "@ " << (i + 1) << ": "<< endl;
				d_fstr_ellip.precision(3);
				d_fstr_ellip << "Read: (x, y, Theta, dx, dy, dTheta) = (" << setw(fwidth) << ellipse->x << ", " << setw(fwidth) << ellipse->y << ", " << setw(fwidth) << ellipse->theta << ", " << setw(fwidth) << trans->tx << ", " << setw(fwidth) << trans->ty << ", " << setw(fwidth) << trans->dTheta << ")" << endl;
				d_fstr_ellip << "Save: (x, y, Theta, dx, dy, dTheta) = (" << setw(fwidth) << tmp.x0     << ", " << setw(fwidth) << tmp.y0     << ", " << setw(fwidth) << tmp.theta      << ", " << setw(fwidth) << tmp.dx    << ", " << setw(fwidth) << tmp.dy    << ", " << setw(fwidth) << tmp.dtheta << ")" << endl;
				d_fstr_ellip << "(x_c, y_c)=>(x_n, y_n): (" << img_dat->regions[id - 1].eCur.x << ", " << img_dat->regions[id - 1].eCur.y << ")=>(" << img_dat->regions[id - 1].eNext.x << ", " << img_dat->regions[id - 1].eNext.y << ")" << endl;
				d_fstr_ellip << endl;
#endif

			}
			else {
				tmp.x0 = 0.f;
				tmp.y0 = 0.f;
				tmp.theta = 0.f;
				tmp.dx = 0.f;
				tmp.dy = 0.f;
				tmp.dtheta = 0.f;


#if DEBUG_SAVE_ELLIPSETRANSF_AS_TEXT == 1
				if(alert_once == true) {
					d_fstr_ellip << "Out of Range." << endl;
					d_fstr_ellip << endl;
					alert_once = false;
				}
#endif
			}

			transport[i] = tmp;
			rt.releaseImageDat(&rt.imgdat);
		}
		else {
			tmp.x0 = 0.f;
			tmp.y0 = 0.f;
			tmp.theta = 0.f;
			tmp.dx = 0.f;
			tmp.dy = 0.f;
			tmp.dtheta = 0.f;
			transport[i] = tmp;

#if DEBUG_SAVE_ELLIPSETRANSF_AS_TEXT == 1
			d_fstr_ellip << " @ " << (i + 1) << ": "<< endl;
			d_fstr_ellip << filename.str().c_str() << ": Not found." << endl;
			d_fstr_ellip << endl;

#endif
		}
	}


#if DEBUG_SAVE_ELLIPSETRANSF_AS_TEXT == 1

	d_fstr_ellip.close();
#endif

	return true;
}


#undef IS_GLOBAL_REGION_ID
#undef DEBUG_SAVE_ELLIPSETRANSF_AS_TEXT
#undef DEBUG_SAVE_ELEMENTS_INFO_AS_TEXT

