#pragma once
#include "StdAfx.h"
#include "RegionLabel.h"


RegionLabel::RegionLabel(void)
: RegionLabelID(NULL)
, RegionLabelIDDefined(false)
, LabelImage(NULL)
, LabelImageDefined(false)
, w(0)
, h(0)
, m(0)
{
}

RegionLabel::~RegionLabel(void) {
	delete[] LabelImage;
	delete[] RegionLabelID;
}

IplImage* RegionLabel::loadIdMap(const char* filename, int* maxId)
{
	//int w,h,m;
	FILE* fp = NULL;
	IplImage* wshed = NULL;
	char *labels = NULL;
	char *lIt;
	int nBytesPerId;
	int *it;

	if(filename<=NULL)
	{
		fprintf(stderr,"In loadIdMap: bad input parameters\n");
		return NULL;
	}

	/* open the file */
	fp = fopen(filename,"rb");
	if(fp<=NULL)
	{
		fprintf(stderr,"In loadIdMap: cannot open the id map file %s\n",filename);
		return NULL;
	}

	/* get the file parameters */
	if(fscanf(fp,"%d %d %d\n",&w,&h,&m)<3)
	{
		fprintf(stderr,"In loadIdMap: cannot read parameters\n");
		fclose(fp);
		return NULL;
	}

	/* create an image to store the data */
	wshed = cvCreateImage( cvSize(w,h), IPL_DEPTH_32S, 1 );
	if(maxId>NULL) *maxId = m;

	if(wshed<=NULL)
	{
		fprintf(stderr,"In loadIdMap: cannot create id map\n");
		fclose(fp);
		return NULL;
	}

	/* read the labels */
	nBytesPerId = (int)(log((double)m)/(8*log(2.0))) + 1;
	lIt = labels = (char*)calloc(w*h,nBytesPerId*sizeof(char));

	if(fread(labels,nBytesPerId*sizeof(char),w*h,fp) != w*h)
	{
		fprintf(stderr,"In loadIdMap: cannot load data from id map file %s\n",filename);
		fclose(fp);
		free(labels);
		cvReleaseImage(&wshed);
		return NULL;
	}

	for(it = (int*)wshed->imageData;it<(int*)(wshed->imageData) + w*h;it++)
	{
		*it = 0;
		memcpy(it,lIt,nBytesPerId*sizeof(char));
		if(*it<=0 || *it>m) *it = 0;
		lIt += nBytesPerId;
	}

	/* free memory and close the file */
	free(labels);

	if(fclose(fp))
	{
		fprintf(stderr,"In loadIdMap: cannot close the id map file %s\n",filename);
	}

	return wshed;
}

void RegionLabel::setRegionLabelID(IplImage* cvImg)
{
	if(RegionLabelIDDefined) delete [] RegionLabelID;
	RegionLabelID = new unsigned int [cvImg->height*cvImg->width];
	for(int i=0;i<cvImg->height;i++)
	{
		for(int j=0;j<cvImg->width;j++)
		{
			RegionLabelID[j + i * cvImg->width]=(unsigned int)((cvGet2D(cvImg,i,j)).val[0]);
		}
	}
	RegionLabelIDDefined=true;

}

void RegionLabel::getLabel( const char* filename)
{
	int maxId;
	int id;
	IplImage* wshed = NULL;

	/* load the .label file */
	wshed = loadIdMap(filename,&maxId);


	if(wshed == NULL) {
		LabelImageDefined = false;
		return;
	}

	/* set RegionLabelID*/
	setRegionLabelID(wshed);

	/* set display image*/
	if(LabelImageDefined) delete [] LabelImage;
	LabelImage = new unsigned char [h*w*3];
	for(int i=0;i<h;i++)
	{
		for(int j=0;j<w;j++)
		{
			
			unsigned int pixel=j + i * w;
            id=RegionLabelID[pixel];
			LabelImage[3*pixel]=63*(id%4 + 1);
			LabelImage[3*pixel+1]=85*(id%3+1);
			LabelImage[3*pixel+2]=255*(id%2);

		}
	}
	
	LabelImageDefined=true;
}

int RegionLabel::getID(int x, int y)
{
	return RegionLabelID[x+y*w];
}
