#pragma once
#include "ILayeredStrokePaintRenderer.h"
#include "SplineRenderer_Extra.h"
#include "StrokeComparator.h"
#include "SimpleStrokeFactory.h"
#include "GLSplineFactory.h"
#include "PaintingStyleField.h"
#include "image_pyramid.h"
#include "glew.h"
#include "GLSLManager.h"




class SplineParam : public IPaintingParam {
public:
	unsigned char *source_img;			// origin at upper left
	unsigned char *stroke_ref_img;		// origin at lower left

	PaintingStyleField *style;
	int time;

	FieldSampleImage *edge_pixel;


	// seeding method
	int seed_order;
	bool partial_ordering;


	int embossing_type;
	GLint *viewport;
	GLdouble *modelview;
	GLdouble *projection;

	SplineParam() {
		source_img = NULL;
		stroke_ref_img = NULL;
		style = NULL;
		time = 0;
		edge_pixel = NULL;
		embossing_type = 0;
		viewport = NULL;
		modelview = NULL;
		projection = NULL;

		seed_order = 0;
		partial_ordering = false;
	}
};



class SplineRenderer : public ILayeredStrokePaintRenderer {
private:
	SplineParam param;
	SimpleStrokeFactory s_stroke_factory;
	GLSplineFactory spline_factory;

	Spline_SimpleLayer **layers;
	unsigned char *ref_image;
	unsigned char *stroke_ref_img_ptr;
	unsigned char *painted_image;
	float *color_noise;


	float g_stroke_spacing;
	int g_stroke_falloff;
	int g_stroke_lifetime;
	int g_stroke_fadetime;
	unsigned char bgfg_threshold;

	int current_layer;
	int num_pixels;


	static const int num_height_patterns = 3;
	GLuint alphamap_handle, heightmap_handle[2][num_height_patterns], *hm_ptr;

	static const float quad_vertex_coords[8];
	static const float quad_texture_coords[8];

	RenderTexture *tmp_target_RT;
	RenderTexture *offscreen;					// offscreen-rendering
	QuadVBO *quad;								// render off-screen texture on screen

	__int64 render_start, render_end;
	__int64 timer_freq;

public:
	SplineRenderer(int width, int height, int num_layers);
	virtual ~SplineRenderer();

	void setParams(IPaintingParam *param);
	bool render(unsigned char *result);
	bool resize(int width, int height);
	bool setNumLayers(int num_layers);

	size_t getNumStrokes(int layer);
	void pushSeeds(int layer, const float *opt_flow, int width, int height);
	void clearSeeds(int layer);

	double getRenderingTime();

	const unsigned char* getInitialImage(int layer);


private:
	bool *initial_image_regeneration;
	GLint prev_tex_env_mode[3];

	void initializeBuffer();
	void finalizeBuffer();

	void setUpMultiTexturePainting();
	void cleanUpMultiTexturePainting();
	void setUpMultiTexturePostProcess();
	void cleanUpMultiTexturePostProcess();

	void makeStrokeTemplate();
	void cleanSeedTemplate();

	void (SplineRenderer::*_renderImage)();
	void createLayeredImage();
	void createHeightImage();
	void traceStrokes(list<SimpleStroke*> &strokes, list<GLSpline*> &splines, float scale, int color_mode = 0, bool render_initial_seeds = false);
	void renderTexturedStrokes(list<GLSpline*> &splines);

	void makeRefImage();
	void regenerateStrokeList();
	void updateStrokeList();
};

