#pragma once
#include "SimpleStroke.h"
#include "RGBComparator.h"
#include "HSVComparator.h"


// sort function used to sort strokes in vector: refer to updateStrokeList()
static inline bool compareStrokeDepth(const SimpleStroke *left, const SimpleStroke *right) {
	return left->depth < right->depth;
}


static inline bool compareStrokeColorMin(const pair< int, SimpleStroke* > &left, const pair< int, SimpleStroke* > &right) {
	int sl, sr;
	const unsigned char *cl = left.second->color, *cr = right.second->color;


	sl = cl[0] * cl[0] + cl[1] * cl[1] + cl[2] * cl[2];
	sr = cr[0] * cr[0] + cr[1] * cr[1] + cr[2] * cr[2];

	if(sl < sr) {
		return true;
	}
	else if(sl > sr) {
		return false;
	}
	else {
		return (cl[1] < cr[1]) ? true : 
			((cl[1] == cr[1] && cl[0] < cr[0]) ? true :
			((cl[1] == cr[1] && cl[0] == cr[0] && cl[2] < cr[2]) ? true : false));
	}
}


static inline bool compareStrokeColorMax(const pair< int, SimpleStroke* > &left, const pair< int, SimpleStroke* > &right) {
	int sl, sr;
	const unsigned char *cl = left.second->color, *cr = right.second->color;


	sl = cl[0] * cl[0] + cl[1] * cl[1] + cl[2] * cl[2];
	sr = cr[0] * cr[0] + cr[1] * cr[1] + cr[2] * cr[2];

	if(sl > sr) {
		return true;
	}
	else if(sl < sr) {
		return false;
	}
	else {
		return (cl[1] > cr[1]) ? true : 
			((cl[1] == cr[1] && cl[0] > cr[0]) ? true :
			((cl[1] == cr[1] && cl[0] == cr[0] && cl[2] > cr[2]) ? true : false));
	}
}


class StrokeComparator {
public:
	static bool compareStrokeDepth(const SimpleStroke *left, const SimpleStroke *right) {
		return left->depth < right->depth;
	}

	static bool compareStrokeRGB(const SimpleStroke *left, const SimpleStroke *right) {
		return rgb_comp.eval(left->color, right->color);
	}

	static bool compareStrokeHSV(const SimpleStroke *left, const SimpleStroke *right) {
		return hsv_comp.eval(left->hsv, right->hsv);
	}

	static RGBComparator rgb_comp;
	static HSVComparator hsv_comp;
};

