#pragma once
#include "icMatrix.h"


class TensorInterpolatable {
public:
	// make everything public...
	static const int num_elems = 2;
	float values[num_elems];
	bool mask;

	TensorInterpolatable() { mask = true; zero(); }
	virtual ~TensorInterpolatable() {}

	// assume 2nd-order symmetric tensor with a form of [a, b; b, -a]
	void pack(const icMatrix2x2 &m) {
		values[0] = (float)m.entry[0][0];
		values[1] = (float)m.entry[1][0];
	}


	void unpack(icMatrix2x2 &m) {
		m.entry[0][0] = values[0];
		m.entry[0][1] = values[1];
		m.entry[1][0] = values[1];
		m.entry[1][1] = -values[0];
	}


	void copy(const TensorInterpolatable &p) {
		*this = p;
		this->mask = p.mask;
	}


	void zero() {
		values[0]	= 0;
		values[1]	= 0;
	}


	// just copy only values. not mask.
	inline TensorInterpolatable &operator=(const TensorInterpolatable &p) {
		this->values[0]		= p.values[0];
		this->values[1]		= p.values[1];

		return *this;	
	}


	inline TensorInterpolatable &operator+=(const TensorInterpolatable &p) {
		this->values[0]		+= p.values[0];
		this->values[1]		+= p.values[1];

		return *this;	
	}


	inline TensorInterpolatable &operator-=(const TensorInterpolatable &p) {
		this->values[0]		-= p.values[0];
		this->values[1]		-= p.values[1];

		return *this;	
	}


	inline TensorInterpolatable &operator*=(const float val) {
		this->values[0]		= this->values[0] * val;
		this->values[1]		= this->values[1] * val;

		return *this;
	}


	inline bool operator<(const TensorInterpolatable &p) {
		return 
			(this->values[0]	>= p.values[0]) ? false : 
			(this->values[1]	>= p.values[1]) ? false : 
			true;
	}


	inline bool operator<(const float &val) {
		return 
			(this->values[0]	>= val) ? false : 
			(this->values[1]	>= val) ? false : 
			true;
	}


	inline bool operator<=(const TensorInterpolatable &p) {
		return 
			(this->values[0]	> p.values[0]) ? false : 
			(this->values[1]	> p.values[1]) ? false : 
			true;
	}


	inline bool operator<=(const float &val) {
		return 
			(this->values[0]	> val) ? false : 
			(this->values[1]	> val) ? false : 
			true;
	}


	inline bool operator>(const TensorInterpolatable &p) {
		return 
			(this->values[0]	<= p.values[0]) ? false : 
			(this->values[1]	<= p.values[1]) ? false : 
			true;
	}


	inline bool operator>(const float &val) {
		return 
			(this->values[0]	<= val) ? false : 
			(this->values[1]	<= val) ? false : 
			true;
	}


	inline bool operator>=(const TensorInterpolatable &p) {
		return 
			(this->values[0]	< p.values[0]) ? false : 
			(this->values[1]	< p.values[1]) ? false : 
			true;
	}


	inline bool operator>=(const float &val) {
		return 
			(this->values[0]	< val) ? false : 
			(this->values[1]	< val) ? false : 
			true;
	}


	inline void _max(const TensorInterpolatable &p) {
#define _ppmax(a, b)            (((a) > (b)) ? (a) : (b))
		this->values[0]		= _ppmax(this->values[0],	p.values[0]);
		this->values[1]		= _ppmax(this->values[1],	p.values[1]);
#undef _ppmax
	}


	inline void _min(const TensorInterpolatable &p) {
#define _ppmin(a, b)            (((a) < (b)) ? (a) : (b))
		this->values[0]		= _ppmin(this->values[0],	p.values[0]);
		this->values[1]		= _ppmin(this->values[1],	p.values[1]);
#undef _ppmin
	}


	inline TensorInterpolatable &operator*=(const double val) {
		return (*this *= (const float)val);
	}


	inline TensorInterpolatable &operator*=(const int val) {
		return (*this *= (const float)val);
	}


	inline const TensorInterpolatable operator+(const TensorInterpolatable &p) const {
		return (TensorInterpolatable(*this) += p);
	}


	inline const TensorInterpolatable operator-(const TensorInterpolatable &p) const {
		return (TensorInterpolatable(*this) -= p);
	}


	inline const TensorInterpolatable operator*(const float val) const {
		return (TensorInterpolatable(*this) *= val);
	}


	inline const TensorInterpolatable operator*(const double val) const {
		return (TensorInterpolatable(*this) *= (const float)val);
	}


	inline const TensorInterpolatable operator*(const int val) const {
		return (TensorInterpolatable(*this) *= (const float)val);
	}


	inline bool operator<(const double &val) { return (*this < (float)val); }
	inline bool operator<=(const double &val) { return (*this <= (float)val); }
	inline bool operator>(const double &val) { return (*this > (float)val); }
	inline bool operator>=(const double &val) { return (*this >= (float)val); }
	
	inline const TensorInterpolatable interpolate(const TensorInterpolatable &operand, float blend_factor) {
		return TensorInterpolatable(*this) * blend_factor + operand * (1.f - blend_factor);
	}
};


inline const TensorInterpolatable operator*(const double val, const TensorInterpolatable &p) { return p * val; }
inline const TensorInterpolatable operator*(const float val, const TensorInterpolatable &p) { return p * val; }
inline const TensorInterpolatable operator*(const int val, const TensorInterpolatable &p) { return p * (float)val; }
