#pragma once
#include "stdafx.h"
#include "Util.h"
#include <math.h>
#include <stdlib.h>
#include <sstream>
#include <iomanip>


using namespace std;



double getNumberFromCEdit(CEdit *ce) {
	double val;
	int buf_len;
	LPTSTR buf;

	buf_len = ce->GetWindowTextLength() + 1;
	buf = new TCHAR[buf_len];
	ce->GetWindowText(buf, buf_len);
	val = _tcstod(buf, NULL);
	delete[] buf;

	return val;
}

void setNumberToCEdit(CEdit *ce, int val) {
#if defined UNICODE
	wstringstream ss;
#else
	stringstream ss;
#endif

	ss << val;
	ce->SetWindowText(ss.str().c_str());
}

void setNumberToCEdit(CEdit *ce, double val, int prec) {
#if defined UNICODE
	wstringstream ss;
#else
	stringstream ss;
#endif

	ss << fixed << setprecision(prec) << val;
	ce->SetWindowText(ss.str().c_str());
}


//
// routine to convert HSV to RGB
//
// Reference:  Foley, van Dam, Feiner, Hughes,
//		"Computer Graphics Principles and Practices,"
//		Additon-Wesley, 1990, pp592-593.
//	       
//
void HsvRgb(float hsv[3], float rgb[3]) {
	float h, s, v;			// hue, sat, value
	float r, g, b;			// red, green, blue
	float i, f, p, q, t;	// interim values


	// guarantee valid input:
	h = hsv[0] / 60.f;
	while(h >= 6.) h -= 6.;
	while(h <  0.) h += 6.;

	s = hsv[1];
	if(s < 0.) s = 0.;
	if(s > 1.) s = 1.;

	v = hsv[2];
	if(v < 0.) v = 0.;
	if(v > 1.) v = 1.;

	// if sat==0, then is a gray:
	if(s == 0.0) {
		rgb[0] = rgb[1] = rgb[2] = v;
		return;
	}

	// get an rgb from the hue itself:	
	i = floor(h);
	f = h - i;
	p = v * (1.f - s);
	q = v * (1.f - s * f);
	t = v * (1.f - (s * (1.f - f)));

	//switch((int) i) {
	switch((int)i % 6) {		// make sure it falls in one of any cases except the default...
		case 0:
			r = v;	g = t;	b = p;
			break;
	
		case 1:
			r = q;	g = v;	b = p;
			break;
	
		case 2:
			r = p;	g = v;	b = t;
			break;
	
		case 3:
			r = p;	g = q;	b = v;
			break;
	
		case 4:
			r = t;	g = p;	b = v;
			break;
	
		case 5:
			r = v;	g = p;	b = q;
			break;

		default:
			//r = 1.f; g = 1.f; b = 1.f;
			//r = 1.f; g = 0.f; b = 0.f;
			r = 0.f; g = 0.f; b = 0.f;

			/**
			FILE *fp = fopen("blah.txt", "a");
			fprintf(fp, "input: hsv: %.3f, %.3f, %.3f\n", hsv[0], hsv[1], hsv[2]);
			fprintf(fp, "computed values: H:%.3f, S:%.3f, V:%.3f, I:%.3f, F:%.3f, P:%.3f, Q:%.3f, T:%.3f\n", h, s, v, i, f, p, q, t);
			fprintf(fp, "\n");
			fclose(fp);
			/**/
			break;
	}

	rgb[0] = r;
	rgb[1] = g;
	rgb[2] = b;
}


//
// routine to convert RGB to HSV
//
// Reference:
//		http://www.cs.rit.edu/~ncs/color/t_convert.html
//	       
//
void RgbHsv(float rgb[3], float hsv[3]) {
	float ch_min, ch_max, delta;

	//ch_min = M_min(M_min(rgb[0], rgb[1]), rgb[2]);
	//ch_max = M_max(M_max(rgb[0], rgb[1]), rgb[2]);
	
	ch_min = rgb[0];
	ch_max = rgb[0];

	for(int i = 1; i < 3; i++) {
		if(ch_min > rgb[i]) ch_min = rgb[i];
		if(ch_max < rgb[i]) ch_max = rgb[i];
	}

	hsv[2] = ch_max;				// v

	delta = ch_max - ch_min;

	if(ch_max != 0) {
		hsv[1] = delta / ch_max;	// s
	}
	else {
		// r = g = b = 0		// s = 0, h is undefined (let it set to 0)
		hsv[1] = 0;
		hsv[0] = -1;
		//hsv[0] = 0;
		return;
	}

	if(rgb[0] == ch_max) {
		hsv[0] = (rgb[1] - rgb[2]) / delta;		// between yellow & magenta
	}
	else if(rgb[1] == ch_max) {
		hsv[0] = 2 + (rgb[2] - rgb[0]) / delta;	// between cyan & yellow
	}
	else {
		hsv[0] = 4 + (rgb[0] - rgb[1]) / delta;	// between magenta & cyan
	}

	hsv[0] *= 60;				// degrees


	while(hsv[0] < 0) {
		hsv[0] += 360;
	}

	while(hsv[0] >= 360) {
		hsv[0] -= 360;
	}

	/**
	if(hsv[0] < 0) {
		hsv[0] += 360;
	}
	/**/
}


/*
	Sort values based on associated keys.
	Both values and keys are sorted.
*/
void QuickSort(int *val, double *key, int low, int high) {
	int i, j, size;
	int tmp_val;
	int pivot;
	double tmp_key;
	

	size = high - low;
	pivot = low + size / 2;
	i = low;
	j = high - 1;

	if(size <= 1) return;
	if(size == 2) {
		if(key[i] > key[j]) {
			tmp_key = key[i];
			key[i] = key[j];
			key[j] = tmp_key;

			tmp_val = val[i];
			val[i] = val[j];
			val[j] = tmp_val;
		}

		return;
	}

	tmp_key = key[pivot];
	key[pivot] = key[j];
	key[j] = tmp_key;

	tmp_val = val[pivot];
	val[pivot] = val[j];
	val[j] = tmp_val;

	pivot = j;
	j--;

	while(i <= j) {
		if(key[i] >= key[pivot]) {
			tmp_val = val[i];
			val[i] = val[j];
			val[j] = tmp_val;

			tmp_key = key[i];
			key[i] = key[j];
			key[j] = tmp_key;

			j--;
		}
		else {
			i++;
		}
	}


	tmp_key = key[i];
	key[i] = key[pivot];
	key[pivot] = tmp_key;

	tmp_val = val[i];
	val[i] = val[pivot];
	val[pivot] = tmp_val;


	QuickSort(val, key, low, i);
	QuickSort(val, key, i + 1, high);

	return;
}


void matrix3X3mult(const double A[9], const double B[9], double ans[9]) {
	for(int i = 0; i < 3; i++) {
		for(int j = 0; j < 3; j++) {
			ans[3 * i + j] = 0;

			for(int k = 0; k < 3; k++) {
				ans[3 * i + j] += A[3 * i + k] * B[3 * k + j];
			}
		}
	}
}


// add outer pixels to input image
// assume 3-channel image...
void expandImage(unsigned char *img, int w, int h, int padding_size, unsigned char **new_img) {
	int nw, nh;
	int idx1, idx2, idx3;

	ASSERT(img);
	ASSERT(new_img);
	ASSERT(w > 0 && h > 0);

	if(padding_size < 0 || padding_size >= w - 1 || padding_size >= h - 1) return;

	nw = w + 2 * padding_size;
	nh = h + 2 * padding_size;
	*new_img = new unsigned char[3 * nw * nh];


	// center
	for(int i = 0; i < h; i++) {
		for(int j = 0; j < w; j++) {
			idx1 = j + w * i;
			idx2 = (j + padding_size) + nw * (i + padding_size);

			for(int k = 0; k < 3; k++) {
				(*new_img)[3 * idx2 + k] = img[3 * idx1 + k];
			}
		}
	}

	// top
	for(int i = 0; i < padding_size; i++) {
		for(int j = 0; j < w; j++) {
			idx1 = j;
			idx2 = idx1 + w * (i + 1);
			idx3 = (j + padding_size) + nw * (padding_size - 1 - i);

			for(int k = 0; k < 3; k++) {
				(*new_img)[3 * idx3 + k] = (unsigned char)::clampVal(2 * img[3 * idx1 + k] - img[3 * idx2 + k], 0, 255);
			}
		}
	}

	// bottom
	for(int i = 0; i < padding_size; i++) {
		for(int j = 0; j < w; j++) {
			idx1 = j + w * (h - 1);
			idx2 = idx1 - w * (i + 1);
			idx3 = (j + padding_size) + nw * (padding_size + h + i);

			for(int k = 0; k < 3; k++) {
				(*new_img)[3 * idx3 + k] = (unsigned char)::clampVal(2 * img[3 * idx1 + k] - img[3 * idx2 + k], 0, 255);
			}
		}
	}

	// left
	for(int i = 0; i < h; i++) {
		for(int j = 0; j < padding_size; j++) {
			idx1 = w * i;
			idx2 = idx1 + j + 1;
			idx3 = (padding_size - 1 - j) + nw * (padding_size + i);

			for(int k = 0; k < 3; k++) {
				(*new_img)[3 * idx3 + k] = (unsigned char)::clampVal(2 * img[3 * idx1 + k] - img[3 * idx2 + k], 0, 255);
			}
		}
	}

	// right
	for(int i = 0; i < h; i++) {
		for(int j = 0; j < padding_size; j++) {
			idx1 = w * (i + 1) - 1;
			idx2 = idx1 + j - 1;
			idx3 = (padding_size + w + j) + nw * (padding_size + i);

			for(int k = 0; k < 3; k++) {
				(*new_img)[3 * idx3 + k] = (unsigned char)::clampVal(2 * img[3 * idx1 + k] - img[3 * idx2 + k], 0, 255);
			}
		}
	}


	// corners
	for(int i = 0; i < padding_size; i++) {
		for(int j = 0; j < padding_size; j++) {
			idx1 = 0;
			idx2 = idx1 + (j + 1) + w * (i + 1);
			idx3 = (padding_size - 1 - j) + nw * (padding_size - 1 - i);

			for(int k = 0; k < 3; k++) {
				(*new_img)[3 * idx3 + k] = (unsigned char)::clampVal(2 * img[3 * idx1 + k] - img[3 * idx2 + k], 0, 255);
			}
		}
	}
 
	for(int i = 0; i < padding_size; i++) {
		for(int j = 0; j < padding_size; j++) {
			idx1 = w - 1;
			idx2 = idx1 - (j + 1) + w * (i + 1);
			idx3 = (padding_size + w + j) + nw * (padding_size - 1 - i);

			for(int k = 0; k < 3; k++) {
				(*new_img)[3 * idx3 + k] = (unsigned char)::clampVal(2 * img[3 * idx1 + k] - img[3 * idx2 + k], 0, 255);
			}
		}
	}

	for(int i = 0; i < padding_size; i++) {
		for(int j = 0; j < padding_size; j++) {
			idx1 = w * (h - 1);
			idx2 = idx1 + (j + 1) - w * (i + 1);
			idx3 = (padding_size - 1 - j) + nw * (padding_size + h + i);

			for(int k = 0; k < 3; k++) {
				(*new_img)[3 * idx3 + k] = (unsigned char)::clampVal(2 * img[3 * idx1 + k] - img[3 * idx2 + k], 0, 255);
			}
		}
	}
 
	for(int i = 0; i < padding_size; i++) {
		for(int j = 0; j < padding_size; j++) {
			idx1 = w * h - 1;
			idx2 = idx1 - (j + 1) - w * (i + 1);
			idx3 = (padding_size + w + j) + nw * (padding_size + h + i);

			for(int k = 0; k < 3; k++) {
				(*new_img)[3 * idx3 + k] = (unsigned char)::clampVal(2 * img[3 * idx1 + k] - img[3 * idx2 + k], 0, 255);
			}
		}
	}
}


// chop off outer pixels of input image
// assume 3-channel image...
void shrinkImage(unsigned char *img, int w, int h, int padding_size, unsigned char **new_img) {
	int idx1, idx2;
	int nw, nh;

	ASSERT(img);
	ASSERT(new_img);
	ASSERT(w > 0 && h > 0);

	nw = w - 2 * padding_size;
	nh = h - 2 * padding_size;

	*new_img = new unsigned char[3 * nw * nh];

	for(int i = 0; i < nh; i++) {
		for(int j = 0; j < nw; j++) {
			idx1 = (padding_size + j) + w * (padding_size + i);
			idx2 = j + nw * i;

			for(int k = 0; k < 3; k++) {
				(*new_img)[3 * idx2 + k] = img[3 * idx1 + k];
			}
		}
	}
}


// assume 3-channel image...
void blurImage(unsigned char *img, int w, int h, int kernel) {
	unsigned char *tmp_img;
	float *tmp_vals;
	float tmp_color[3];
	int half_kernel;
	int nw, nh;
	int idx1, idx2;
	int kernel_squared;

	ASSERT(img);
	ASSERT(w > 0 && h > 0);

	if(kernel <= 1) return;

	// clamp to some odd number
	if(kernel == ((kernel >> 1) << 1)) kernel++;

	half_kernel = kernel >> 1;
	nh = h + 2 * half_kernel;
	nw = w + 2 * half_kernel;
	kernel_squared = kernel * kernel;

	expandImage(img, w, h, half_kernel, &tmp_img);
	tmp_vals = new float[3 * w * nh];
	memset(tmp_vals, 0, sizeof(float) * 3 * w * nh);


	for(int i = 0; i < nh; i++) {
		for(int j = 0; j < w; j++) {
			idx1 = j + w * i;
			
			for(int m = 0; m < kernel; m++) {
				idx2 = (j + m) + nw * i;

				for(int k = 0; k < 3; k++) {
					tmp_vals[3 * idx1 + k] += tmp_img[3 * idx2 + k];
				}
			}
		}
	}

	for(int i = 0; i < w; i++) {
		for(int j = 0; j < h; j++) {
			idx1 = j + w * i;

			for(int k = 0; k < 3; k++) {
				tmp_color[k] = 0;
			}

			for(int m = 0; m < kernel; m++) {
				idx2 = j + w * (m + i);

				for(int k = 0; k < 3; k++) {
					tmp_color[k] += tmp_vals[3 * idx2 + k];
				}
			}

			for(int k = 0; k < 3; k++) {
				img[3 * idx1 + k] = (unsigned char)::clampVal(tmp_color[k] / kernel_squared, 0, 255);
			}
		}
	}

	delete[] tmp_img;
	delete[] tmp_vals;
}


bool findRegion(int w, int h, int x0, int y0, int kernel, int *xs, int *ys, bool clamp) {
	int half_kernel;
	int idx, x_start, y_start;
	int w_1, h_1;

	ASSERT(w > 0 && h > 0);
	ASSERT(kernel > 0);

	if(kernel == (kernel >> 1) << 1) return false;

	half_kernel = kernel >> 1;

	x_start = x0 - half_kernel;
	y_start = y0 - half_kernel;
	w_1 = w - 1;
	h_1 = h - 1;


	if(clamp == true) {
		idx = 0;

		for(int i = 0; i < kernel; i++) {
			for(int j = 0; j < kernel; j++) {
				xs[idx] = (int)::clampVal(x_start + j, 0, w_1);
				ys[idx] = (int)::clampVal(y_start + i, 0, h_1);

				idx++;
			}
		}
	}
	else {
		idx = 0;

		for(int i = 0; i < kernel; i++) {
			for(int j = 0; j < kernel; j++) {
				xs[idx] = x_start + j;
				ys[idx] = y_start + i;

				idx++;
			}
		}
	}

	return true;
}


// assume images are forming 3-channel component...
void computeColorDifference(unsigned char *img1, unsigned char *img2, int w, int h, float *diff) {
	int num_pixels, idx;
	int d[3];
	float one_third = 1.f / 3.f;
	unsigned char *p1, *p2;

	num_pixels = w * h;

	idx = 0;

	for(int i = 0; i < num_pixels; i++, idx += 3) {
		p1 = &(img1[idx]);
		p2 = &(img2[idx]);
		d[0] = p1[0] - p2[0];
		d[1] = p1[1] - p2[1];
		d[2] = p1[2] - p2[2];

		// scaled in range [0, 255]
		diff[i] = sqrt((d[0] * d[0] + d[1] * d[1] + d[2] * d[2]) * one_third);
		// scaled in range [0, 255^2]
		//diff[i] = (d[0] * d[0] + d[1] * d[1] + d[2] * d[2]) * one_third;
	}
}

