#pragma once
#include <math.h>
#include <utility>

#define M_PI 3.1415926535


using namespace std;

const double pi = M_PI;
const double rad = pi / 180.;
const double inv_rad = 180. / pi;


inline double degreeToRadian(double deg) {
	return deg * rad;
}

inline double radianToDegree(double radian) {
	return radian * inv_rad;
}

inline double inverse_dist(double len, double threshold) {
	if(len < threshold) len = threshold;
	return 1.0 / (len * len);
}


inline double clampVal(double val, double min, double max) {
	return (val < min) ? min : ((val > max) ? max : val);
}


inline double calcSquaredIntensity(unsigned char pixel[3]) {
	return (pixel[0] * pixel[0] + pixel[1] * pixel[1] + pixel[2] * pixel[2]) / 3.;
}


inline double gauss(double len) {	
	if (len < 1.0e-6)
		len = 1.0e-6;
	return exp(-len*len*100.0);
}



inline unsigned char interpolate(unsigned char a, unsigned char b, unsigned char s) {
	return (unsigned char)((a * s) / 255 + (b * (255 - s)) / 255);
}


inline double calcIntensity(unsigned char pixel[3]) { return sqrt(calcSquaredIntensity(pixel)); }
inline double magnitude(double x, double y) { return sqrt(x * x + y * y); }
inline double magnitude(double x, double y, double z) { return sqrt(x * x + y * y + z * z); }




void HsvRgb(float hsv[3], float rgb[3]);
void RgbHsv(float rgb[3], float hsv[3]);

// maybe dengerous since input is assumed to be supplied (no NULL passing)
inline void RgbHsv(unsigned char rgb[3], float hsv[3]) {
	const float frac = 1.f / 255.f;
	float rgb_f[3] = {rgb[0] * frac, rgb[1] * frac, rgb[2] * frac};

	RgbHsv(rgb_f, hsv);
}

inline void HsvRgb(float hsv[3], unsigned char rgb[3]) {
	float rgb_f[3];

	HsvRgb(hsv, rgb_f);
	rgb[0] = (unsigned char)(rgb_f[0] * 255);
	rgb[1] = (unsigned char)(rgb_f[1] * 255);
	rgb[2] = (unsigned char)(rgb_f[2] * 255);
}


void QuickSort(int *val, double *key, int low, int high);
void matrix3X3mult(const double A[9], const double B[9], double ans[9]);

void expandImage(unsigned char *img, int w, int h, int padding_size, unsigned char **new_img);
void shrinkImage(unsigned char *img, int w, int h, int padding_size, unsigned char **new_img);
void blurImage(unsigned char *img, int w, int h, int kernel);
bool findRegion(int w, int h, int x0, int y0, int kernel, int *xs, int *ys, bool clamp = true);



inline int computeSquaredColorDifference(unsigned char c0[3], unsigned char c1[3]) {
	int d0 = c0[0] - c1[0];
	int d1 = c0[1] - c1[1];
	int d2 = c0[2] - c1[2];

	return d0 * d0 + d1 * d1 + d2 * d2;
}


void computeColorDifference(unsigned char *img1, unsigned char *img2, int w, int h, float *diff);


double getNumberFromCEdit(CEdit *ce);
void setNumberToCEdit(CEdit *ce, int val);
void setNumberToCEdit(CEdit *ce, double val, int prec);


template <class T> T M_max(T left, T right) {
	if(left > right) return left;
	else return right;
}


template <class T> T M_min(T left, T right) {
	if(left < right) return left;
	else return right;
}


template<class A, class B> inline bool firstElemIsLess(const pair<A, B> &left, const pair<A, B> &right) {
	return left.first < right.first;
}


inline float calcAlpha(float alpha_max, float falloff, int lifetime, int fadetime, int t) {
	int half_lifetime = lifetime + fadetime;
	int total_lifetime = half_lifetime + fadetime;
	float alpha_min = alpha_max * falloff;
	float scaled_time;


	if(0 < t && t <= fadetime) {
		scaled_time = (float)t / fadetime;

		return alpha_max * ((1 - scaled_time) * falloff + scaled_time);
	}
	else if(fadetime < t && t <= half_lifetime) {
		return alpha_max;
	}
	else if(half_lifetime < t && t <= total_lifetime) {
		scaled_time = (float)(t - half_lifetime) / fadetime;

		return alpha_max * (scaled_time * falloff + 1.f - scaled_time);
	}
	else {
		return 0;
	}
}

