#pragma once
#include "StdAfx.h"
#include "glSpline.h"
#include "GLSLManager.h"
#include "shader_flag.h"
#include "renderer_debug.h"


const float tip = 1.f / 8.f;




GLSpline::GLSpline(float x0, float y0, float width, float *color, float angle, bool use_texture) : SplineStroke(x0, y0, width, color, angle, 1) {
//GLSpline::GLSpline(float x0, float y0, float width, float *color, float angle, bool use_texture) : SplineStroke(x0, y0, width, color, angle, 0) {
	rec_coords = NULL;
	height_tex_coords = NULL;
	alpha_tex_coords = NULL;
	num_vertices = 0;
	alpha_tex_handle = 0;
	height_tex_handle = 0;
	height_tex_height_scale = 1.f;
	height_tex_coord_scale = 1.f;
	depth = 0;

	vertices_capacity = 0;

	black[0] = black[1] = black[2] = 0;
	black[3] = this->color[3];

	setColorMode(COLOR);
	useTexture(use_texture);
}


GLSpline::~GLSpline() {
	if(rec_coords != NULL) delete[] rec_coords;
	if(height_tex_coords != NULL) delete[] height_tex_coords;
	if(alpha_tex_coords != NULL) delete[] alpha_tex_coords;
}


bool GLSpline::tesselate(bool add_ends) {
	int idx = 0, idx2 = 0;
	int &nsp = num_spline_points;		// short hand notation...
	int num_segments;
	float hw = stroke_size * .5f;
	float *&sp = spline_points;


	tesselated = false;
	num_vertices = 0;


	if(SplineStroke::tesselate(add_ends) == true) {
		int l_idx = 2 * nsp - 1;
		int num_inner_points = nsp - 2 * (density + 1);
		float val, delta = 0, tmp;


		tesselated = false;
		num_segments = nsp - 1;
		num_vertices = 2 * nsp;

		if(rec_coords == NULL || alpha_tex_coords == NULL) {	// ....
			vertices_capacity = num_vertices;
			rec_coords = new float[2 * vertices_capacity];
			alpha_tex_coords = new float[2 * vertices_capacity];

#if USE_GLSL_SPLINE == 0
			height_tex_coords = new float[2 * vertices_capacity];
#endif

		}
		else if(vertices_capacity < num_vertices) {
			delete[] rec_coords;
			delete[] alpha_tex_coords;
			delete[] height_tex_coords;

			vertices_capacity = num_vertices;
			rec_coords = new float[2 * vertices_capacity];
			alpha_tex_coords = new float[2 * vertices_capacity];

#if USE_GLSL_SPLINE == 0
			height_tex_coords = new float[2 * vertices_capacity];
#endif

		}

		if(num_segments > 0) {
			computeSegment(sp[0], sp[1], sp[2], sp[3], sp[0], sp[1], hw, &(rec_coords[0]));

			idx = 2;
			idx2 = 4;

			for(int i = 1; i < num_segments; i++) {
				computeSegment(sp[idx - 2], sp[idx - 1], sp[idx + 2], sp[idx + 3], sp[idx], sp[idx + 1], hw, &(rec_coords[idx2]));

				idx += 2;
				idx2 += 4;
			}

			computeSegment(sp[l_idx - 3], sp[l_idx - 2], sp[l_idx - 1], sp[l_idx], sp[l_idx - 1], sp[l_idx], hw, &(rec_coords[idx2]));


			if(num_inner_points > 1) {
				idx = 0;

				delta = tip / (density + 1);

				for(int i = 0; i <= density; i++) {
					val = i * delta;

					alpha_tex_coords[idx++] = val;
					alpha_tex_coords[idx++] = 1.f;
					alpha_tex_coords[idx++] = val;
					alpha_tex_coords[idx++] = 0;
				}


				delta = (1.f - 2 * tip) / (num_inner_points - 1);

				for(int i = 0; i < num_inner_points; i++) {
					val = tip + i * delta;

					alpha_tex_coords[idx++] = val;
					alpha_tex_coords[idx++] = 1.f;
					alpha_tex_coords[idx++] = val;
					alpha_tex_coords[idx++] = 0;
				}


				delta = tip / (density + 1);
				tmp = 1.f - tip;

				for(int i = 0; i <= density; i++) {
					val = tmp + (i + 1) * delta;

					alpha_tex_coords[idx++] = val;
					alpha_tex_coords[idx++] = 1.f;
					alpha_tex_coords[idx++] = val;
					alpha_tex_coords[idx++] = 0;
				}
			}
			else {
				idx = 0;
				delta = 1.f / num_segments;

				for(int i = 0; i <= num_segments; i++) {
					val = i * delta;

					alpha_tex_coords[idx++] = val;
					alpha_tex_coords[idx++] = 1.f;
					alpha_tex_coords[idx++] = val;
					alpha_tex_coords[idx++] = 0;
				}
			}

			tesselated = true;
		}
	}

	return tesselated;
}


bool GLSpline::drawWithTexture(void *params) {
	glColor4fv(color_ptr);

#if USE_GLSL_SPLINE == 1
	GLSLManager::SetTextureToIdx(0, alpha_tex_handle);
#else
	glBindTexture(GL_TEXTURE_2D, alpha_tex_handle);
#endif

	glTexCoordPointer(2, GL_FLOAT, 0, alpha_tex_coords);
	glVertexPointer(2, GL_FLOAT, 0, rec_coords);


#if D_USE_DEPTH_SPLINE == 0
	glDrawArrays(GL_TRIANGLE_STRIP, 0, num_vertices);

#else
	glPushMatrix();
	glTranslatef(0, 0, -depth);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, num_vertices);
	glPopMatrix();

#endif


#if USE_GLSL_SPLINE == 1
	GLSLManager::ResetTextures();
#else
	glBindTexture(GL_TEXTURE_2D, 0);
#endif

	return true;
}


bool GLSpline::drawHeightMap(void *params) {
	int idx = 0;
	float offset;
	float tex_env_color[4];
	//float offset = *(float*)(params);

	if(params != NULL) {
		offset = *(float*)(params);
		//glColor4f(offset, offset, offset, 1.f);
		glColor4f(offset, offset, offset, color_ptr[3]);
	}
	else {
		glColor4fv(color_ptr);
	}


#if USE_GLSL_SPLINE == 1
	GLSLManager::SetTextureToIdx(0, height_tex_handle);
	GLSLManager::SetTextureToIdx(1, alpha_tex_handle);
	glTexCoordPointer(2, GL_FLOAT, 0, alpha_tex_coords);

#else
	
	tex_env_color[0] = height_tex_height_scale;
	tex_env_color[1] = height_tex_height_scale;
	tex_env_color[2] = height_tex_height_scale;
	tex_env_color[3] = 1.f;

	idx = 0;

	for(int i = 0; i < num_vertices; i++) {
		height_tex_coords[idx] = alpha_tex_coords[idx] * height_tex_coord_scale; idx++;
		height_tex_coords[idx] = alpha_tex_coords[idx] * height_tex_coord_scale; idx++;
	}


	glActiveTexture(GL_TEXTURE0);
	glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, tex_env_color);
	glBindTexture(GL_TEXTURE_2D, height_tex_handle);

	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, alpha_tex_handle);

	glActiveTexture(GL_TEXTURE2);
	glBindTexture(GL_TEXTURE_2D, alpha_tex_handle);


	glClientActiveTexture(GL_TEXTURE0);
	glTexCoordPointer(2, GL_FLOAT, 0, height_tex_coords);

	glClientActiveTexture(GL_TEXTURE1);
	glTexCoordPointer(2, GL_FLOAT, 0, alpha_tex_coords);

	glClientActiveTexture(GL_TEXTURE2);
	glTexCoordPointer(2, GL_FLOAT, 0, alpha_tex_coords);

#endif

	glVertexPointer(2, GL_FLOAT, 0, rec_coords);


#if D_USE_DEPTH_SPLINE == 0
	glDrawArrays(GL_TRIANGLE_STRIP, 0, num_vertices);

#else
	glPushMatrix();
	glTranslatef(0, 0, -depth);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, num_vertices);
	glPopMatrix();

#endif



#if USE_GLSL_SPLINE == 1
	GLSLManager::ResetTextures();
#else
	glActiveTexture(GL_TEXTURE2);
	glBindTexture(GL_TEXTURE_2D, 0);
	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, 0);
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, 0);

#endif

	return true;
}


bool GLSpline::drawWithColor(void *params) {		// this is for debug purpose...
	glColor4fv(color_ptr);
	glVertexPointer(2, GL_FLOAT, 0, rec_coords);


#if D_USE_DEPTH_SPLINE == 0
	glDrawArrays(GL_TRIANGLE_STRIP, 0, num_vertices);

#else
	glPushMatrix();
	glTranslatef(0, 0, -depth);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, num_vertices);
	glPopMatrix();

#endif

	return true;
}


void GLSpline::reset(float x0, float y0, float width, float *color, float angle, bool use_texture) {
	SplineStroke::reset(x0, y0, width, color, angle, this->density);

	num_vertices = 0;
	alpha_tex_handle = 0;
	height_tex_handle = 0;
	black[3] = this->color[3];
	useTexture(use_texture);
	depth = 0;
}

#undef USE_GLSL_SPLINE
