#pragma once
#include "SplineStroke.h"
//#include <gl\gl.h>
#include "GL/glew.h"
#include <math.h>


class GLSpline : public SplineStroke {
public:
	enum COLOR_MODE {
		COLOR, BLACK
	};

	GLSpline(float x0, float y0, float width, float *color, float angle, bool use_texture = false);
	virtual ~GLSpline();
	virtual bool draw(void *params = NULL) { return (this->*_drawColor)(params); }
	virtual bool drawHeight(void *params = NULL) { return (this->*_drawHeight)(params); }
	virtual bool tesselate(bool add_ends = true);

	inline void useTexture(bool val) { 
		use_texture = val;  
		_drawColor = val ? &GLSpline::drawWithTexture : &GLSpline::drawWithColor;
		_drawHeight = val ? &GLSpline::drawHeightMap : &GLSpline::drawWithColor;
	}

	inline bool useTextureEnabled() { return use_texture; }

	inline void setAlphaTexture(GLuint tex_handle = 0) { alpha_tex_handle = tex_handle; }
	inline GLuint getAlphaTexture() { return alpha_tex_handle; }
	inline void setHeightMapTexture(GLuint tex_handle = 0)  { height_tex_handle = tex_handle; }
	inline GLuint getHeightMapTexture() { return height_tex_handle; }
	inline void setHeightMapTexHeightScale(float scale) { height_tex_height_scale = scale; }
	inline void setHeightMapTexCoordScale(float scale) { height_tex_coord_scale = scale; }

	void reset(float x0, float y0, float width, float *color, float angle, bool use_texture = false);

	inline void setDepth(float z) { depth = z; }
	inline float getDepth() { return depth; }

	inline void setColor(float color[4]) {
		this->color[0] = color[0];
		this->color[1] = color[1];
		this->color[2] = color[2];
		this->color[3] = color[3];
		this->black[3] = color[3];
	}

	inline void setColor(float r, float g, float b, float a) {
		this->color[0] = r;
		this->color[1] = g;
		this->color[2] = b;
		this->color[3] = a;
		this->black[3] = a;
	}

	inline void setColorMode(GLSpline::COLOR_MODE mode) {
		current_color = mode;

		switch(mode) {
			case COLOR:
				color_ptr = color;
				break;

			case BLACK:
				color_ptr = black;
				break;
		}
	}

	inline GLSpline::COLOR_MODE getColorMode() { return current_color; }


protected:
	GLuint alpha_tex_handle, height_tex_handle;
	bool use_texture;
	float *alpha_tex_coords;
	float *height_tex_coords;
	float *rec_coords;
	int num_vertices;
	int vertices_capacity;
	float height_tex_height_scale;
	float height_tex_coord_scale;

	float black[4];
	float *color_ptr;
	float depth;

	bool (GLSpline::*_drawColor)(void *params);
	bool (GLSpline::*_drawHeight)(void *params);

	COLOR_MODE current_color;


private:
	inline void computeSegment(float x0, float y0, float x1, float y1, float xc, float yc, float hw, float *v_buffer) {
		float dx, dy, sf;

		// already rotate by 90 degrees.
		dx = y0 - y1;
		dy = x1 - x0;

		// assume that the length is non-zero
		sf = hw / sqrt(dx * dx + dy * dy);
		dx *= sf;
		dy *= sf;

		v_buffer[0] = xc + dx;
		v_buffer[1] = yc + dy;
		v_buffer[2] = xc - dx;
		v_buffer[3] = yc - dy;
	}

	bool drawWithColor(void *params);
	bool drawWithTexture(void *params);			// use GL_MODLATE for texture environment
	bool drawHeightMap(void *param);			// use GL_ADD for texture environment
};
