#include <math.h>
#define GOLD 1.618034
#define GLIMIT 200.0
#define TINY 1.0e-20
#define ITMAX 100 // 100
#define CGOLD 0.3819660
#define ZEPS 1.0e-10
#define TOL 2.0e-4
#define SHFT(a,b,c,d) (a)=(b);(b)=(c);(c)=(d);
//#define J

icVector2 pcom, xicom;
double (*nrfunc)(icVector2, int, int, int);

// this one is used to find optimal square width for patch packing
void mnbrak(double *ax, double *bx, double *cx, double *fa, double *fb, double *fc, double (*func)(double))
{
	double ulim,u,r,q,fu,dum;

	*fa=(*func)(*ax);
	*fb=(*func)(*bx);
	if (*fb>*fa){
		SHFT(dum,*ax,*bx,dum)
		SHFT(dum,*fb,*fa,dum)
	}
	*cx=(*bx)+GOLD*(*bx-*ax);
	*fc=(*func)(*cx);
	while (*fb > *fc) {
		r=(*bx-*ax)*(*fb-*fc);
		q=(*bx-*cx)*(*fb-*fa);
		if (fabs(q-r)<TINY) {
			if (q-r>=0.0)
				u=(*bx)-((*bx-*cx)*q-(*bx-*ax)*r)/(2.0*TINY);
			else
				u=(*bx)-((*bx-*cx)*q-(*bx-*ax)*r)/(-2.0*TINY);
		} else {
			if (q-r>=0.0)
				u=(*bx)-((*bx-*cx)*q-(*bx-*ax)*r)/(2.0*(q-r));
			else
				u=(*bx)-((*bx-*cx)*q-(*bx-*ax)*r)/(-2.0*(q-r));
		}
		ulim=(*bx)+GLIMIT*(*cx-*bx);
		if ((*bx-u)*(u-*cx)>0.0) {
			fu=(*func)(u);
			if (fu<*fc) {
				*ax=(*bx);
				*bx=u;
				*fa=(*fb);
				*fb=fu;
				return;
			} else if (fu>*fb) {
				*cx=u;
				*fc=fu;
				return;
			}
			u=(*cx)+GOLD*(*cx-*bx);
			fu=(*func)(u);
		} else if ((*cx-u)*(u-ulim)>0.0) {
			fu=(*func)(u);
			if (fu<*fc){
				SHFT(*bx, *cx, u, *cx+GOLD*(*cx-*bx))
				SHFT(*fb,*fc,fu,(*func)(u))
			}
		} else if ((u-ulim)*(ulim-*cx)>=0.0) {
			u=ulim;
			fu=(*func)(u);
		} else {
			u=(*cx)+GOLD*(*cx-*bx);
			fu=(*func)(u);
		}
		SHFT(*ax,*bx,*cx,u)
		SHFT(*fa,*fb,*fc,fu)
	}
}

double brent(double ax, double bx, double cx, double (*f)(double), double tol, double *xmin)
{
	int iter;
	double a, b, d, etemp, fu, fv, fw, fx, p, q, r, tol1, tol2, u, v, w, x, xm;
	double e=0.0;

	a=(ax<cx?ax:cx);
	b=(ax>cx?ax:cx);
	x=w=v=bx;
	fw=fv=fx=(*f)(x);
	for (iter=1;iter<=ITMAX;iter++) {
		xm=0.5*(a+b);
		tol2=2.0*(tol1=tol*fabs(x)+ZEPS);
		if (fabs(x-xm)<=(tol2-0.5*(b-a))){
			fx = (*f)(x);
			*xmin=x;
			return fx;
		}
		if (fabs(e)>tol1) {
			r=(x-w)*(fx-fv);
			q=(x-v)*(fx-fw);
			p=(x-v)*q-(x-w)*r;
			q=2.0*(q-r);
			if (q>0.0) p = -p;
			q=fabs(q);
			etemp=e;
			e=d;
			if ((fabs(p)>=fabs(0.5*q*etemp)) || (p<=q*(a-x)) || (p>=q*(b-x)))
				d=CGOLD*(e=(x>=xm?a-x:b-x));
			else {
				d=p/q;
				u=x+d;
				if (u-a<tol2 || b-u<tol2) {
					if (xm>=x)
						d=tol1;
					else
						d=-tol1;
				}
			}
		} else {
			d=CGOLD*(e=(x>=xm?a-x:b-x));
		}
		if (d>=0.0)
			u=(fabs(d)>=tol1?x+d:x+tol1);
		else
			u=(fabs(d)>=tol1?x+d:x-tol1);
		fu=(*f)(u);
		if (fu<=fx) {
			if (u>=x) a=x; else b=x;
			SHFT(v,w,x,u);
			SHFT(fv,fw,fx,fu);
		} else {
			if (u<x) a=u; else b=u;
			if (fu<=fw || w==x) {
				v=w;
				w=u;
				fv=fw;
				fw=fu;
			} else if (fu<=fv || v==x || v==w) {
				v=u;
				fv=fu;
			}
		}
	}
	fprintf(stderr, "Too many iterations in brent\n");
	*xmin=x;
	return fx;
}

// below are used for line searching for stretch optimization
void mnbrak(double *ax, double *bx, double *cx, double *fa, double *fb, double *fc, double (*func)(double, int, int, int), int patchid, int vertid, int normid)
{
	double ulim,u,r,q,fu,dum;

	*fa=(*func)(*ax, patchid, vertid, normid);
	*fb=(*func)(*bx, patchid, vertid, normid);
	if (*fb>*fa){
		SHFT(dum,*ax,*bx,dum)
		SHFT(dum,*fb,*fa,dum)
	}
	*cx=(*bx)+GOLD*(*bx-*ax);
	*fc=(*func)(*cx, patchid, vertid, normid);
	while (*fb > *fc) {
		r=(*bx-*ax)*(*fb-*fc);
		q=(*bx-*cx)*(*fb-*fa);
		if (fabs(q-r)<TINY) {
			if (q-r>=0.0)
				u=(*bx)-((*bx-*cx)*q-(*bx-*ax)*r)/(2.0*TINY);
			else
				u=(*bx)-((*bx-*cx)*q-(*bx-*ax)*r)/(-2.0*TINY);
		} else {
			if (q-r>=0.0)
				u=(*bx)-((*bx-*cx)*q-(*bx-*ax)*r)/(2.0*(q-r));
			else
				u=(*bx)-((*bx-*cx)*q-(*bx-*ax)*r)/(-2.0*(q-r));
		}
		ulim=(*bx)+GLIMIT*(*cx-*bx);
		if ((*bx-u)*(u-*cx)>0.0) {
			fu=(*func)(u, patchid, vertid, normid);
			if (fu<*fc) {
				*ax=(*bx);
				*bx=u;
				*fa=(*fb);
				*fb=fu;
				return;
			} else if (fu>*fb) {
				*cx=u;
				*fc=fu;
				return;
			}
			u=(*cx)+GOLD*(*cx-*bx);
			fu=(*func)(u, patchid, vertid, normid);
		} else if ((*cx-u)*(u-ulim)>0.0) {
			fu=(*func)(u, patchid, vertid, normid);
			if (fu<*fc){
				SHFT(*bx, *cx, u, *cx+GOLD*(*cx-*bx))
				SHFT(*fb,*fc,fu,(*func)(u, patchid, vertid, normid))
			}
		} else if ((u-ulim)*(ulim-*cx)>=0.0) {
			u=ulim;
			fu=(*func)(u, patchid, vertid, normid);
		} else {
			u=(*cx)+GOLD*(*cx-*bx);
			fu=(*func)(u, patchid, vertid, normid);
		}
		SHFT(*ax,*bx,*cx,u)
		SHFT(*fa,*fb,*fc,fu)
	}
}

double brent(double ax, double bx, double cx, double (*f)(double, int, int, int), double tol, double *xmin, int patchid, int vertid, int normid)
{
	int iter;
	double a, b, d, etemp, fu, fv, fw, fx, p, q, r, tol1, tol2, u, v, w, x, xm;
	double e=0.0;

	a=(ax<cx?ax:cx);
	b=(ax>cx?ax:cx);
	x=w=v=bx;
	fw=fv=fx=(*f)(x, patchid, vertid, normid);
	for (iter=1;iter<=ITMAX;iter++) {
		xm=0.5*(a+b);
		tol2=2.0*(tol1=tol*fabs(x)+ZEPS);
		if (fabs(x-xm)<=(tol2-0.5*(b-a))){
			*xmin=x;
			return fx;
		}
		if (fabs(e)>tol1) {
			r=(x-w)*(fx-fv);
			q=(x-v)*(fx-fw);
			p=(x-v)*q-(x-w)*r;
			q=2.0*(q-r);
			if (q>0.0) p = -p;
			q=fabs(q);
			etemp=e;
			e=d;
			if ((fabs(p)>=fabs(0.5*q*etemp)) || (p<=q*(a-x)) || (p>=q*(b-x)))
				d=CGOLD*(e=(x>=xm?a-x:b-x));
			else {
				d=p/q;
				u=x+d;
				if (u-a<tol2 || b-u<tol2) {
					if (xm>=x)
						d=tol1;
					else
						d=-tol1;
				}
			}
		} else {
			d=CGOLD*(e=(x>=xm?a-x:b-x));
		}
		if (d>=0.0)
			u=(fabs(d)>=tol1?x+d:x+tol1);
		else
			u=(fabs(d)>=tol1?x+d:x-tol1);
		fu=(*f)(u, patchid, vertid, normid);
		if (fu<=fx) {
			if (u>=x) a=x; else b=x;
			SHFT(v,w,x,u);
			SHFT(fv,fw,fx,fu);
		} else {
			if (u<x) a=u; else b=u;
			if (fu<=fw || w==x) {
				v=w;
				w=u;
				fv=fw;
				fw=fu;
			} else if (fu<=fv || v==x || v==w) {
				v=u;
				fv=fu;
			}
		}
	}
	fprintf(stderr, "Too many iterations in brent\n");
	*xmin=x;
	return fx;
}

double fidim(double x, int patchid, int vertid, int normid)
{
	double f;
	icVector2 xt;

	xt=pcom+x*xicom;
	f=(*nrfunc)(xt, patchid, vertid, normid);
	return f;
}

void linmin(icVector2 *p, icVector2 *xi, double *fret, double (*func)(icVector2, int, int, int), int patchid, int vertid, double max_distance, int normid)
{
	double xx, xmin, fx, fb, fa, bx, ax;

	nrfunc = func;
	pcom = (*p)*1.0;
	xicom = (*xi)*1.0;
	ax=0.0;
	xx=max_distance;
	mnbrak(&ax, &xx, &bx, &fa, &fx, &fb, fidim, patchid, vertid, normid);
	*fret=brent(ax, xx, bx, fidim, TOL, &xmin, patchid, vertid, normid);
	(*xi) *= xmin;
	(*p) += (*xi);
}

